"use strict";

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

var _Control = _interopRequireDefault(require("./Control"));

var _core = require("./consts/core");

var _controls = require("./consts/controls");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Control spec', function () {
  it('renders with default values', function () {
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Control.default, null));
    var divs = wrapper.find('div');
    expect(divs).toHaveLength(1);
    expect(divs.children).toHaveLength(1);
    expect(divs.contains(_core.DEFAULT_VALUE)).toBeTruthy();
    expect(divs.hasClass(_controls.CONTROL)).toBeTruthy();
  });
  it('renders div with provided value', function () {
    var value = '<';
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Control.default, {
      value: value
    }));
    expect(wrapper.find('div').contains(value));
  });
  it('sets data-attributes and classNames dependin on provided props', function () {
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Control.default, {
      disabled: true
    }));
    var divs = wrapper.find('div');
    var props = divs.props();
    expect(divs.hasClass(_core.DISABLED)).toBeTruthy();
    expect(props['data-disabled']).toBeTruthy();
  });
  it('sets classname', function () {
    var myClassName = 'myClassName';
    var myOtherClassName = 'myOtherClassName';
    var className = "".concat(myClassName, " ").concat(myOtherClassName);
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Control.default, {
      className: className
    }));
    var divs = wrapper.find('div');
    expect(divs.hasClass(myClassName)).toBeTruthy();
    expect(divs.hasClass(myOtherClassName)).toBeTruthy();
  });
  it('should invoke provided handler', function () {
    var value = 12;
    var handler = jest.fn();
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Control.default, {
      value: value,
      onClick: handler
    }));
    wrapper.find('div').simulate('click');
    expect(handler).toHaveBeenCalledTimes(1);
    expect(handler).toHaveBeenCalledWith(value);
  });
  it('should not invoke provided handler if disabled', function () {
    var handler = jest.fn();
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Control.default, {
      onClick: handler,
      disabled: true
    }));
    wrapper.find('div').simulate('click');
    expect(handler).not.toHaveBeenCalled();
  });
});