"use strict";

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

var _Input = _interopRequireDefault(require("./Input"));

var _core = require("./consts/core");

var _input = require("./consts/input");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Input spec', function () {
  it('renders input with default value', function () {
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Input.default, null));
    var inputs = wrapper.find('input');
    var props = inputs.props();
    expect(inputs).toHaveLength(1);
    expect(inputs.hasClass(_input.VALID)).toBeTruthy();
    expect(inputs.hasClass(_input.INPUT)).toBeTruthy();
    expect(props['data-valid']).toBeTruthy();
    expect(props.value).toBe(_core.DEFAULT_VALUE);
  });
  it('should call validator, when trigger input change', function () {
    var validator = jest.fn();
    var simulatedValue = 12;
    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Input.default, {
      validator: validator
    }));
    var inputs = wrapper.find('input');
    expect(validator).toHaveBeenNthCalledWith(1, _core.DEFAULT_VALUE);
    inputs.simulate('change', {
      target: {
        value: simulatedValue
      }
    });
    expect(validator).toHaveBeenCalledTimes(2);
    expect(validator).toHaveBeenNthCalledWith(2, simulatedValue);
  });
  it('should call provided handler, if value is not same', function () {
    var onChange = jest.fn();
    var simulatedValue = 12;
    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Input.default, {
      onChange: onChange
    }));
    var inputs = wrapper.find('input');
    inputs.simulate('change', {
      target: {
        value: simulatedValue
      }
    });
    expect(onChange).toHaveBeenCalledTimes(1);
    expect(onChange).toHaveBeenCalledWith(simulatedValue);
  });
  it('should not call provided handler if values are same', function () {
    var onChange = jest.fn();
    var simulatedValue = _core.DEFAULT_VALUE;
    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Input.default, {
      onChange: onChange
    }));
    var inputs = wrapper.find('input');
    inputs.simulate('change', {
      target: {
        value: simulatedValue
      }
    });
    expect(onChange).not.toHaveBeenCalled();
  });
  it('should not call provided handler if validator fails', function () {
    var onChange = jest.fn();

    var validator = function validator() {
      return false;
    };

    var simulatedValue = _core.DEFAULT_VALUE;
    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Input.default, {
      onChange: onChange,
      validator: validator
    }));
    var inputs = wrapper.find('input');
    inputs.simulate('change', {
      target: {
        value: simulatedValue
      }
    });
    expect(onChange).not.toHaveBeenCalled();
  });
  it('should update ui after setState', function () {
    var simulatedValue = 12;
    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Input.default, {
      validator: function validator(val) {
        return val < simulatedValue;
      }
    }));
    var inputs = wrapper.find('input');
    inputs.simulate('change', {
      target: {
        value: simulatedValue
      }
    });
    expect(inputs.hasClass(_input.INVALID)).toBeTruthy();
    expect(inputs.hasClass(_input.VALID)).toBeFalsy();
  });
  it('should add className', function () {
    var one = 'one';
    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Input.default, {
      className: one
    }));
    expect(wrapper.find('input').hasClass(one));
  });
});