"use strict";

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

var _Picker = _interopRequireDefault(require("./Picker"));

var _core = require("./consts/core");

var _picker = require("./consts/picker");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Picker spec', function () {
  it('renders div with default value', function () {
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Picker.default, null));
    var divs = wrapper.find('div');
    expect(divs).toHaveLength(1);
    expect(divs.children).toHaveLength(1);
    expect(divs.contains(_core.DEFAULT_VALUE)).toBeTruthy();
    expect(divs.hasClass(_picker.PICKER)).toBeTruthy();
  });
  it('renders div with provided value', function () {
    var value = 'First';
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Picker.default, {
      value: value
    }));
    expect(wrapper.find('div').contains(value));
  });
  it('sets data-attributes and classNames dependin on provided props', function () {
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Picker.default, {
      disabled: true,
      picked: true
    }));
    var divs = wrapper.find('div');
    var props = divs.props();
    expect(divs.hasClass(_core.DISABLED)).toBeTruthy();
    expect(divs.hasClass(_core.PICKED)).toBeTruthy();
    expect(props['data-picked']).toBeTruthy();
    expect(props['data-disabled']).toBeTruthy();
  });
  it('sets classname', function () {
    var myClassName = 'myClassName';
    var myOtherClassName = 'myOtherClassName';
    var className = "".concat(myClassName, " ").concat(myOtherClassName);
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Picker.default, {
      className: className
    }));
    var divs = wrapper.find('div');
    expect(divs.hasClass(myClassName)).toBeTruthy();
    expect(divs.hasClass(myOtherClassName)).toBeTruthy();
  });
  it('should invoke provided handler', function () {
    var value = 12;
    var onClick = jest.fn();
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Picker.default, {
      value: value,
      onClick: onClick
    }));
    wrapper.find('div').simulate('click');
    expect(onClick).toHaveBeenCalledTimes(1);
    expect(onClick).toHaveBeenCalledWith(value);
  });
  it('should not invoke provided handler if disabled', function () {
    var onClick = jest.fn();
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Picker.default, {
      onClick: onClick,
      disabled: true
    }));
    wrapper.find('div').simulate('click');
    expect(onClick).not.toHaveBeenCalled();
  });
});