"use strict";

var _functional = require("./functional");

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('functional utils spec', function () {
  describe('noop', function () {
    it('should return undefind', function () {
      expect((0, _functional.noop)()).toBeUndefined();
    });
  });
  describe('identity', function () {
    it('should return provided argument', function () {
      expect((0, _functional.identity)()).toBeUndefined();
      expect((0, _functional.identity)(12)).toBe(12);
      var a = {
        a: 12
      };
      expect((0, _functional.identity)(a)).toBe(a);
    });
  });
  describe('pipe', function () {
    it('should return identity if no funcs provided', function () {
      var identity = (0, _functional.pipe)();
      expect(identity).toBeInstanceOf(Function);
      expect(identity(12)).toBe(12);
    });
    it('should return provided func, if there are only one func provided', function () {
      var func = function func(val) {
        return val + 12;
      };

      var f = (0, _functional.pipe)(func);
      expect(f).toBe(func);
      expect(f(12)).toBe(func(12));
    });
    it('should create flow of function invoke', function () {
      var addA = function addA(val) {
        return function (obj) {
          return _objectSpread({}, obj, {
            a: val
          });
        };
      };

      var addB = function addB(val) {
        return function (obj) {
          return _objectSpread({}, obj, {
            b: val
          });
        };
      };

      var addC = function addC(val) {
        return function (obj) {
          return _objectSpread({}, obj, {
            c: val
          });
        };
      };

      var addMultipleValues = (0, _functional.pipe)(addA(1), addB(2), addC(3));
      expect(addMultipleValues({})).toEqual({
        a: 1,
        b: 2,
        c: 3
      });
    });
  });
  describe('assert', function () {
    it('should throw error if condition is false', function () {
      var f = function f() {
        return (0, _functional.assert)(false, 'NO');
      };

      expect(f).toThrowError('Assertion failed: NO');
    });
    it('should return condition if condition is truthy', function () {
      expect((0, _functional.assert)(true, 'NO')).toBe(true);
    });
  });
});