"use strict";

var _paginator = require("./paginator");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var wrapPicker = function wrapPicker(value) {
  return {
    value: value,
    disabled: false,
    picked: false
  };
};

var wrapPickers = function wrapPickers() {
  var values = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  return values.map(wrapPicker);
};

describe('paginator utils spec', function () {
  describe('isValueInvalid', function () {
    it('should return true if value is invalid', function () {
      expect((0, _paginator.isValueInvalid)(0)).toBeTruthy();
      expect((0, _paginator.isValueInvalid)(-1)).toBeTruthy();
      expect((0, _paginator.isValueInvalid)(null)).toBeTruthy();
      expect((0, _paginator.isValueInvalid)(undefined)).toBeTruthy();
      expect((0, _paginator.isValueInvalid)(false)).toBeTruthy();
    });
    it('should return false if value is valid', function () {
      expect((0, _paginator.isValueInvalid)(1)).toBeFalsy();
      expect((0, _paginator.isValueInvalid)(123)).toBeFalsy();
      expect((0, _paginator.isValueInvalid)(Infinity)).toBeFalsy();
      expect((0, _paginator.isValueInvalid)(true)).toBeFalsy();
    });
  });
  describe('calculateAllAvailablePickers', function () {
    it('should return 0, if provided values is not valid', function () {
      expect((0, _paginator.calculateAllAvailablePickers)(0, 12)).toBe(0);
      expect((0, _paginator.calculateAllAvailablePickers)(12, 0)).toBe(0);
    });
    it('should retun ceil value, as a result of division', function () {
      var all = 90;
      var perPage = 12;
      var expected = Math.ceil(all / perPage);
      expect((0, _paginator.calculateAllAvailablePickers)(all, perPage)).toBe(expected);
    });
  });
  describe('calculateVisiblePickers', function () {
    it('should return 0 if provided values are not valid', function () {
      expect((0, _paginator.calculateVisiblePickers)(0, 12)).toBe(0);
      expect((0, _paginator.calculateVisiblePickers)(12, 0)).toBe(0);
    });
    it('should return value - how many pickers to show, depend on arguments', function () {
      expect((0, _paginator.calculateVisiblePickers)(123, 12)).toBe(12);
      expect((0, _paginator.calculateVisiblePickers)(10, 12)).toBe(10);
      expect((0, _paginator.calculateVisiblePickers)(12, 12)).toBe(12);
      expect((0, _paginator.calculateVisiblePickers)(1, 12)).toBe(1);
      expect((0, _paginator.calculateVisiblePickers)(100, 99)).toBe(99);
    });
  });
  describe('createSequence', function () {
    it('should return empty array, if provided value is not valid', function () {
      expect((0, _paginator.createSequence)()).toEqual([]);
      expect((0, _paginator.createSequence)(null, null)).toEqual([]);
      expect((0, _paginator.createSequence)(NaN, NaN)).toEqual([]);
      expect((0, _paginator.createSequence)(NaN, 12)).toEqual([]);
      expect((0, _paginator.createSequence)(12, null)).toEqual([]);
    });
    it('should return sequence, depending on provided values', function () {
      expect((0, _paginator.createSequence)(0, 5)).toEqual([0, 1, 2, 3, 4, 5]);
      expect((0, _paginator.createSequence)(10, 15)).toEqual([10, 11, 12, 13, 14, 15]);
      expect((0, _paginator.createSequence)(-1, 3)).toEqual([-1, 0, 1, 2, 3]);
      expect((0, _paginator.createSequence)(-10, -7)).toEqual([-10, -9, -8, -7]);
      expect((0, _paginator.createSequence)(3, 0)).toEqual([3]);
      expect((0, _paginator.createSequence)(0, -10)).toEqual([0]);
    });
    it('should return sequence, with provided step value', function () {
      expect((0, _paginator.createSequence)(0, 5, 2)).toEqual([0, 2, 4]);
      expect((0, _paginator.createSequence)(0, 5, 5)).toEqual([0, 5]);
      expect((0, _paginator.createSequence)(0, 100, 20)).toEqual([0, 20, 40, 60, 80, 100]);
      expect((0, _paginator.createSequence)(0, 99, 20)).toEqual([0, 20, 40, 60, 80]);
      expect((0, _paginator.createSequence)(0, -20, -20)).toEqual([0, -20]);
      expect((0, _paginator.createSequence)(0, -34, -12)).toEqual([0, -12, -24]);
    });
  });
  describe('calculateIndexesUp', function () {
    it('should throw an error, if value is not valid', function () {
      expect(function () {
        return (0, _paginator.calculateIndexesUp)(12, 5, 6);
      }).toThrow();
    });
    it('should return sequence with full visible count, if it is include last value', function () {
      expect((0, _paginator.calculateIndexesUp)(8, 4, 12)).toEqual([8, 9, 10, 11, 12]);
      expect((0, _paginator.calculateIndexesUp)(9, 4, 12)).toEqual([8, 9, 10, 11, 12]);
      expect((0, _paginator.calculateIndexesUp)(10, 4, 12)).toEqual([8, 9, 10, 11, 12]);
      expect((0, _paginator.calculateIndexesUp)(11, 4, 12)).toEqual([8, 9, 10, 11, 12]);
      expect((0, _paginator.calculateIndexesUp)(12, 4, 12)).toEqual([8, 9, 10, 11, 12]);
      expect((0, _paginator.calculateIndexesUp)(1, 4, 5)).toEqual([1, 2, 3, 4, 5]);
      expect((0, _paginator.calculateIndexesUp)(2, 4, 5)).toEqual([1, 2, 3, 4, 5]);
      expect((0, _paginator.calculateIndexesUp)(3, 4, 5)).toEqual([1, 2, 3, 4, 5]);
      expect((0, _paginator.calculateIndexesUp)(4, 4, 5)).toEqual([1, 2, 3, 4, 5]);
      expect((0, _paginator.calculateIndexesUp)(5, 4, 5)).toEqual([1, 2, 3, 4, 5]);
    });
    it('should return correct sequence, depending on visible count -1', function () {
      expect((0, _paginator.calculateIndexesUp)(1, 4, 12)).toEqual([1, 2, 3, 4]);
      expect((0, _paginator.calculateIndexesUp)(2, 4, 12)).toEqual([2, 3, 4, 5]);
      expect((0, _paginator.calculateIndexesUp)(3, 4, 12)).toEqual([3, 4, 5, 6]);
      expect((0, _paginator.calculateIndexesUp)(4, 4, 12)).toEqual([4, 5, 6, 7]);
      expect((0, _paginator.calculateIndexesUp)(5, 4, 12)).toEqual([5, 6, 7, 8]);
      expect((0, _paginator.calculateIndexesUp)(6, 4, 12)).toEqual([6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesUp)(7, 4, 12)).toEqual([7, 8, 9, 10]);
      expect((0, _paginator.calculateIndexesUp)(12, 3, 18)).toEqual([12, 13, 14]);
      expect((0, _paginator.calculateIndexesUp)(13, 3, 18)).toEqual([13, 14, 15]);
      expect((0, _paginator.calculateIndexesUp)(14, 3, 18)).toEqual([14, 15, 16]);
    });
  });
  describe('calculateIndexesDown', function () {
    it("should return sequence from 1 to visible count - 1,\n\t\t\tif current index less then visible count -1", function () {
      expect((0, _paginator.calculateIndexesDown)(1, 4, 5)).toEqual([1, 2, 3, 4]);
      expect((0, _paginator.calculateIndexesDown)(2, 4, 5)).toEqual([1, 2, 3, 4]);
      expect((0, _paginator.calculateIndexesDown)(3, 4, 5)).toEqual([1, 2, 3, 4]);
      expect((0, _paginator.calculateIndexesDown)(4, 4, 5)).toEqual([1, 2, 3, 4]);
      expect((0, _paginator.calculateIndexesDown)(1, 9, 15)).toEqual([1, 2, 3, 4, 5, 6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesDown)(2, 9, 15)).toEqual([1, 2, 3, 4, 5, 6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesDown)(3, 9, 15)).toEqual([1, 2, 3, 4, 5, 6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesDown)(4, 9, 15)).toEqual([1, 2, 3, 4, 5, 6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesDown)(5, 9, 15)).toEqual([1, 2, 3, 4, 5, 6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesDown)(6, 9, 15)).toEqual([1, 2, 3, 4, 5, 6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesDown)(7, 9, 15)).toEqual([1, 2, 3, 4, 5, 6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesDown)(8, 9, 15)).toEqual([1, 2, 3, 4, 5, 6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesDown)(9, 9, 15)).toEqual([1, 2, 3, 4, 5, 6, 7, 8, 9]);
    });
    it("should invoke calculateIndexesUp, \n\t\tso that current index should be last value in array, or include last value", function () {
      expect((0, _paginator.calculateIndexesDown)(5, 4, 15)).toEqual([2, 3, 4, 5]);
      expect((0, _paginator.calculateIndexesDown)(6, 4, 15)).toEqual([3, 4, 5, 6]);
      expect((0, _paginator.calculateIndexesDown)(7, 4, 15)).toEqual([4, 5, 6, 7]);
      expect((0, _paginator.calculateIndexesDown)(8, 4, 15)).toEqual([5, 6, 7, 8]);
      expect((0, _paginator.calculateIndexesDown)(9, 4, 15)).toEqual([6, 7, 8, 9]);
      expect((0, _paginator.calculateIndexesDown)(10, 4, 15)).toEqual([7, 8, 9, 10]);
      expect((0, _paginator.calculateIndexesDown)(11, 4, 15)).toEqual([8, 9, 10, 11]);
      expect((0, _paginator.calculateIndexesDown)(12, 4, 15)).toEqual([9, 10, 11, 12]);
      expect((0, _paginator.calculateIndexesDown)(13, 4, 15)).toEqual([10, 11, 12, 13]);
      expect((0, _paginator.calculateIndexesDown)(14, 4, 15)).toEqual([11, 12, 13, 14, 15]);
      expect((0, _paginator.calculateIndexesDown)(15, 4, 15)).toEqual([11, 12, 13, 14, 15]);
      expect((0, _paginator.calculateIndexesDown)(3, 2, 5)).toEqual([2, 3]);
      expect((0, _paginator.calculateIndexesDown)(4, 2, 5)).toEqual([3, 4, 5]);
      expect((0, _paginator.calculateIndexesDown)(5, 2, 5)).toEqual([3, 4, 5]);
    });
  });
  describe('calculateIndexes', function () {
    it('should return empty array if provided values is not valid', function () {
      expect((0, _paginator.calculateIndexes)({
        currentIndex: -1
      })).toEqual([]);
      expect((0, _paginator.calculateIndexes)({})).toEqual([]);
      expect((0, _paginator.calculateIndexes)({
        all: 0
      })).toEqual([]);
      expect((0, _paginator.calculateIndexes)({
        visibleAmount: 0
      })).toEqual([]);
      expect((0, _paginator.calculateIndexes)({
        up: true
      })).toEqual([]);
    });
    it('should throw an error if both directions are provided', function () {
      expect(function () {
        return (0, _paginator.calculateIndexes)({
          up: true,
          down: true
        });
      }).toThrow();
    });
    it("should return array of one item, or from 1 to 2,\n\t\tif visible amount is less or equal 2", function () {
      expect((0, _paginator.calculateIndexes)({
        visibleAmount: 1,
        currentIndex: 1,
        all: 20
      })).toEqual([1]);
      expect((0, _paginator.calculateIndexes)({
        visibleAmount: 2,
        currentIndex: 1,
        all: 20
      })).toEqual([1, 2]);
    });
    it('should use up direction by default', function () {
      expect((0, _paginator.calculateIndexes)({
        visibleAmount: 5,
        currentIndex: 1,
        all: 20
      })).toEqual([1, 2, 3, 4]);
      expect((0, _paginator.calculateIndexes)({
        visibleAmount: 5,
        currentIndex: 2,
        all: 20
      })).toEqual([2, 3, 4, 5]);
      expect((0, _paginator.calculateIndexes)({
        visibleAmount: 5,
        currentIndex: 3,
        all: 20
      })).toEqual([3, 4, 5, 6]);
    });
    it('should call calculate indexes, depending on provided direction', function () {
      var calcIndexUp = jest.fn();
      var calIndexDown = jest.fn();
      expect((0, _paginator.calculateIndexes)({
        visibleAmount: 5,
        currentIndex: 1,
        all: 20,
        up: true
      })).toEqual([1, 2, 3, 4]);
      (0, _paginator.calculateIndexes)({
        visibleAmount: 5,
        currentIndex: 1,
        all: 20,
        up: true
      }, calcIndexUp, calIndexDown);
      expect(calcIndexUp).toHaveBeenCalledTimes(1);
      expect(calcIndexUp).toHaveBeenCalledWith(1, 4, 20);
      expect((0, _paginator.calculateIndexes)({
        visibleAmount: 5,
        currentIndex: 6,
        all: 20,
        down: true
      })).toEqual([3, 4, 5, 6]);
      (0, _paginator.calculateIndexes)({
        visibleAmount: 5,
        currentIndex: 6,
        all: 20,
        down: true
      }, calcIndexUp, calIndexDown);
      expect(calIndexDown).toHaveBeenCalledTimes(1);
      expect(calIndexDown).toHaveBeenCalledWith(6, 4, 20);
    });
  });
  describe('addValue', function () {
    it('should return function', function () {
      var f = (0, _paginator.addValue)();
      expect(f).toBeInstanceOf(Function);
    });
    it('should return new array, if values provided', function () {
      var arr = [12, 12];
      expect((0, _paginator.addValue)(13)(arr)).not.toBe(arr);
    });
    it('should return old arr, if no values provided', function () {
      var arr = [12, 12];
      expect((0, _paginator.addValue)()(arr)).toBe(arr);
    });
    it('should add new value', function () {
      expect((0, _paginator.addValue)(12)([13])).toEqual([13, 12]);
      expect((0, _paginator.addValue)(12)([])).toEqual([12]);
      expect((0, _paginator.addValue)(12)([13, 14])).toEqual([13, 14, 12]);
      expect((0, _paginator.addValue)(12)()).toEqual([12]);
    });
  });
  describe('addValues', function () {
    it('should return function', function () {
      var f = (0, _paginator.addValues)();
      expect(f).toBeInstanceOf(Function);
    });
    it('should return new array, if values provided', function () {
      var arr = [12, 12];
      expect((0, _paginator.addValues)([13])(arr)).not.toBe(arr);
    });
    it('should return old arr, if no values provided', function () {
      var arr = [12, 12];
      expect((0, _paginator.addValues)()(arr)).toBe(arr);
    });
    it('should concat provided values, to existing array', function () {
      expect((0, _paginator.addValues)([1, 2, 3])([5])).toEqual([5, 1, 2, 3]);
      expect((0, _paginator.addValues)([1, 2, 3, 4, 5])([5, 6])).toEqual([5, 6, 1, 2, 3, 4, 5]);
      expect((0, _paginator.addValues)()()).toEqual([]);
      expect((0, _paginator.addValues)([12])()).toEqual([12]);
      expect((0, _paginator.addValues)()([12])).toEqual([12]);
      expect((0, _paginator.addValues)([])([12])).toEqual([12]);
      expect((0, _paginator.addValues)([12])([])).toEqual([12]);
    });
  });
  describe('producePickerMap', function () {
    it('should throw an error, if lastIndex is not provided', function () {
      expect(function () {
        return (0, _paginator.producePickerMap)();
      }).toThrow();
    });
    it('should return new array', function () {
      expect((0, _paginator.producePickerMap)({
        lastIndex: 12
      })).toEqual([]);
    });
    it('should return array with pickers, depending on provided options', function () {
      var indexes = [1, 2, 3, 4, 5];
      var firstLabel = 'FIRST';
      var lastLabel = 'LAST';
      var onlyFirstLable = {
        firstLabel: firstLabel
      };
      var onlyLastLabel = {
        lastLabel: lastLabel
      };
      var labels = {
        firstLabel: firstLabel,
        lastLabel: lastLabel
      };
      var controlUp = '>';
      var controlDown = '<';
      var onlyUpConrol = {
        controlUp: controlUp
      };
      var onlyDownControl = {
        controlDown: controlDown
      };
      var controls = {
        controlUp: controlUp,
        controlDown: controlDown
      };
      var withLast = true;
      var lastIndex = 6;
      var delimeter = '...'; //only indexes

      var onlyIndexes = (0, _paginator.producePickerMap)({
        indexes: indexes,
        lastIndex: lastIndex
      });
      var expectedOnlyIndexes = wrapPickers(indexes);
      expect(onlyIndexes).toEqual(expectedOnlyIndexes); // indexes, first label

      var indexesAndFirstLabel = (0, _paginator.producePickerMap)({
        indexes: indexes,
        labels: onlyFirstLable,
        lastIndex: lastIndex
      });
      var expectedIndexesAndFirstLabel = wrapPickers([firstLabel].concat(indexes));
      expect(indexesAndFirstLabel).toEqual(expectedIndexesAndFirstLabel); // indexes, last label

      var indexesAndLastLabels = (0, _paginator.producePickerMap)({
        indexes: indexes,
        labels: onlyLastLabel,
        lastIndex: lastIndex
      });
      var expectedIndexesAndLastLabels = wrapPickers(indexes.concat([lastLabel]));
      expect(indexesAndLastLabels).toEqual(expectedIndexesAndLastLabels); //indexes, all labels

      var indexesAndAllLabels = (0, _paginator.producePickerMap)({
        indexes: indexes,
        labels: labels,
        lastIndex: lastIndex
      });
      var expectedIndexesAndAllLabels = wrapPickers([firstLabel].concat(indexes, [lastLabel]));
      expect(indexesAndAllLabels).toEqual(expectedIndexesAndAllLabels); //indexes, controlUp

      var indexesAndControlUp = (0, _paginator.producePickerMap)({
        indexes: indexes,
        controls: onlyUpConrol,
        lastIndex: lastIndex
      });
      var expectedIndexesAndControlUp = wrapPickers(indexes.concat([controlUp]));
      expect(indexesAndControlUp).toEqual(expectedIndexesAndControlUp); //indexes, controlDown

      var indexesAndContolDown = (0, _paginator.producePickerMap)({
        indexes: indexes,
        controls: onlyDownControl,
        lastIndex: lastIndex
      });
      var expectedIndexesAndContolDown = wrapPickers([controlDown].concat(indexes));
      expect(indexesAndContolDown).toEqual(expectedIndexesAndContolDown); //indexes, all controls

      var indexesAndAllControls = (0, _paginator.producePickerMap)({
        indexes: indexes,
        controls: controls,
        lastIndex: lastIndex
      });
      var expectedIndexesAndAllControls = wrapPickers([controlDown].concat(indexes, [controlUp]));
      expect(indexesAndAllControls).toEqual(expectedIndexesAndAllControls); //indexes, labels, controls

      var indexesAndAllLabelsAndAllControls = (0, _paginator.producePickerMap)({
        indexes: indexes,
        controls: controls,
        labels: labels,
        lastIndex: lastIndex
      });
      var expectedIndexesAndAllLabelsAndAllControls = [expectedIndexesAndAllLabels[0], expectedIndexesAndAllControls[0]].concat(_toConsumableArray(wrapPickers(indexes)), [expectedIndexesAndAllControls[expectedIndexesAndAllControls.length - 1], expectedIndexesAndAllLabels[expectedIndexesAndAllLabels.length - 1]]);
      expect(indexesAndAllLabelsAndAllControls).toEqual(expectedIndexesAndAllLabelsAndAllControls); //with last no delimeter

      var withLastNoDelimeter = (0, _paginator.producePickerMap)({
        indexes: indexes,
        controls: controls,
        labels: labels,
        withLast: withLast,
        lastIndex: lastIndex
      });
      var expectedWithLastNoDelimeter = [expectedIndexesAndAllLabels[0], expectedIndexesAndAllControls[0]].concat(_toConsumableArray(wrapPickers(indexes.concat([lastIndex]))), [expectedIndexesAndAllControls[expectedIndexesAndAllControls.length - 1], expectedIndexesAndAllLabels[expectedIndexesAndAllLabels.length - 1]]);
      expect(withLastNoDelimeter).toEqual(expectedWithLastNoDelimeter); //with last, delimeter, last index

      var withLastAndDelimeter = (0, _paginator.producePickerMap)({
        indexes: indexes,
        controls: controls,
        labels: labels,
        withLast: withLast,
        lastIndex: lastIndex,
        delimeter: delimeter
      });
      var expectedWithLastAndDelimeter = [expectedIndexesAndAllLabels[0], expectedIndexesAndAllControls[0]].concat(_toConsumableArray(wrapPickers(indexes.concat([delimeter, lastIndex]))), [expectedIndexesAndAllControls[expectedIndexesAndAllControls.length - 1], expectedIndexesAndAllLabels[expectedIndexesAndAllLabels.length - 1]]);
      expect(withLastAndDelimeter).toEqual(expectedWithLastAndDelimeter); //with currentIndex first

      var withCurrentIndexOne = (0, _paginator.producePickerMap)({
        indexes: indexes,
        controls: controls,
        labels: labels,
        withLast: withLast,
        lastIndex: lastIndex,
        delimeter: delimeter,
        currentIndex: 1
      });
      var expectedWithCurrentIndex = [expectedIndexesAndAllLabels[0], expectedIndexesAndAllControls[0]].concat(_toConsumableArray(wrapPickers(indexes.concat([delimeter, lastIndex]))), [expectedIndexesAndAllControls[expectedIndexesAndAllControls.length - 1], expectedIndexesAndAllLabels[expectedIndexesAndAllLabels.length - 1]]);
      expectedWithCurrentIndex[0].disabled = true;
      expectedWithCurrentIndex[1].disabled = true;
      expectedWithCurrentIndex[2].picked = true;
      expect(withCurrentIndexOne).toEqual(expectedWithCurrentIndex); //with current index last

      var withCurrentIndexTwo = (0, _paginator.producePickerMap)({
        indexes: indexes,
        controls: controls,
        labels: labels,
        withLast: withLast,
        lastIndex: lastIndex,
        delimeter: delimeter,
        currentIndex: indexes[indexes.length - 1]
      });
      var expectedWithCurrentIndexTwo = [expectedIndexesAndAllLabels[0], expectedIndexesAndAllControls[0]].concat(_toConsumableArray(wrapPickers(indexes.concat([delimeter, lastIndex]))), [expectedIndexesAndAllControls[expectedIndexesAndAllControls.length - 1], expectedIndexesAndAllLabels[expectedIndexesAndAllLabels.length - 1]]);
      expectedWithCurrentIndexTwo[expectedWithCurrentIndexTwo.length - 1].disabled = true;
      expectedWithCurrentIndexTwo[expectedWithCurrentIndexTwo.length - 2].disabled = true;
      expectedWithCurrentIndexTwo[expectedWithCurrentIndexTwo.length - 3].picked = true;
      expect(withCurrentIndexTwo).toEqual(withCurrentIndexTwo);
    });
  });
});