"use strict";

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

var _Input = _interopRequireDefault(require("./Input"));

var _Control = _interopRequireDefault(require("./Control"));

var _Label = _interopRequireDefault(require("./Label"));

var _Picker = _interopRequireDefault(require("./Picker"));

var _Delimeter = _interopRequireDefault(require("./Delimeter"));

var _Paginator = _interopRequireDefault(require("./Paginator"));

var _paginator = require("./consts/paginator");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

var getCustomComponent = function getCustomComponent() {
  return (
    /*#__PURE__*/
    function (_React$Component) {
      _inherits(CustomComponent, _React$Component);

      function CustomComponent() {
        _classCallCheck(this, CustomComponent);

        return _possibleConstructorReturn(this, _getPrototypeOf(CustomComponent).apply(this, arguments));
      }

      _createClass(CustomComponent, [{
        key: "render",
        value: function render() {
          return _react.default.createElement("div", null);
        }
      }]);

      return CustomComponent;
    }(_react.default.Component)
  );
};

var checkProps = function checkProps(props, expected) {
  Object.keys(expected).forEach(function (name) {
    var actual = props[name];
    var real = expected[name];
    expect(actual).toBe(real);
  });
};

describe('Paginator specs', function () {
  it('renders components with default values', function () {
    var amount = 190;
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Paginator.default, {
      amount: amount
    }));
    var divs = wrapper.find('div');
    expect(wrapper.find(_Input.default)).toHaveLength(1);
    expect(wrapper.find(_Delimeter.default)).toHaveLength(1);
    expect(wrapper.find(_Control.default)).toHaveLength(2);
    expect(wrapper.find(_Label.default)).toHaveLength(2);
    expect(wrapper.find(_Picker.default)).toHaveLength(_paginator.DEFAULT_AMOUNT_PICKERS_TO_SHOW);
    expect(divs.first().hasClass(_paginator.PAGINATOR)).toBeTruthy();
  });
  it('disables parts, when disable provided', function () {
    var amount = 120;
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Paginator.default, {
      amount: amount
    }));
    wrapper.setProps({
      enableControls: false
    });
    expect(wrapper.find(_Control.default)).toHaveLength(0);
    wrapper.setProps({
      enableInputControl: false
    });
    expect(wrapper.find(_Input.default)).toHaveLength(0);
    wrapper.setProps({
      enableLabels: false
    });
    expect(wrapper.find(_Label.default)).toHaveLength(0);
    wrapper.setProps({
      enableDelimeter: false
    });
    expect(wrapper.find(_Delimeter.default)).toHaveLength(0);
  });
  it('renders custom components, if they are provided', function () {
    var amount = 120;
    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Paginator.default, {
      amount: amount
    }));
    var customPickerComponent = getCustomComponent();
    wrapper.setProps({
      customPickerComponent: customPickerComponent
    });
    expect(wrapper.find(customPickerComponent)).toHaveLength(_paginator.DEFAULT_AMOUNT_PICKERS_TO_SHOW);
    expect(wrapper.find(_Picker.default)).toHaveLength(0);
    var customDelimeterComponent = getCustomComponent();
    wrapper.setProps({
      customDelimeterComponent: customDelimeterComponent
    });
    expect(wrapper.find(customDelimeterComponent)).toHaveLength(1);
    expect(wrapper.find(_Delimeter.default)).toHaveLength(0);
    var customLabelComponent = getCustomComponent();
    wrapper.setProps({
      customLabelComponent: customLabelComponent
    });
    expect(wrapper.find(customLabelComponent)).toHaveLength(2);
    expect(wrapper.find(_Label.default)).toHaveLength(0);
    var customControlComponent = getCustomComponent();
    wrapper.setProps({
      customControlComponent: customControlComponent
    });
    expect(wrapper.find(customControlComponent)).toHaveLength(2);
    expect(wrapper.find(_Control.default)).toHaveLength(0);
    var customInputComponent = getCustomComponent();
    wrapper.setProps({
      customInputComponent: customInputComponent
    });
    expect(wrapper.find(customInputComponent)).toHaveLength(1);
    expect(wrapper.find(_Input.default)).toHaveLength(0);
  });
  it('should provide props to components', function () {
    var amount = 120;
    var controlsProps = {
      controlUp: '+',
      controlDown: '-',
      controlClassName: 'control'
    };
    var inputProps = {
      inputControlValidator: function inputControlValidator() {},
      inputClassName: 'input'
    };
    var labelsProps = {
      firstLabel: 'F',
      lastLabel: 'L',
      labelClassName: 'label'
    };
    var delimeterProps = {
      delimeterValue: '++++',
      delimeterClassName: 'delimeter'
    };
    var pickerProps = {
      pickerClassName: 'picker'
    };

    var props = _objectSpread({}, controlsProps, inputProps, labelsProps, delimeterProps, pickerProps);

    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Paginator.default, _extends({
      amount: amount
    }, props)));
    var controls = wrapper.find(_Control.default);
    var firstControl = controls.at(0);
    checkProps(firstControl.props(), {
      value: controlsProps.controlDown,
      className: controlsProps.controlClassName
    });
    var secondControl = controls.at(1);
    checkProps(secondControl.props(), {
      value: controlsProps.controlUp,
      className: controlsProps.controlClassName
    });
    var input = wrapper.find(_Input.default);
    checkProps(input.props(), {
      validator: inputProps.inputControlValidator,
      className: inputProps.inputClassName
    });
    var labels = wrapper.find(_Label.default);
    var firstLabel = labels.at(0);
    checkProps(firstLabel.props(), {
      value: labelsProps.firstLabel,
      className: labelsProps.labelClassName
    });
    var secondLabel = labels.at(1);
    checkProps(secondLabel.props(), {
      value: labelsProps.lastLabel,
      className: labelsProps.labelClassName
    });
    var delimeter = wrapper.find(_Delimeter.default);
    checkProps(delimeter.props(), {
      value: delimeterProps.delimeterValue,
      className: delimeterProps.delimeterClassName
    });
    var pickers = wrapper.find(_Picker.default);
    pickers.forEach(function (picker) {
      checkProps(picker.props(), {
        className: pickerProps.pickerClassName
      });
    });
  });
  it('should call provided handler with proper value', function () {
    var amount = 120;
    var handler = jest.fn();
    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Paginator.default, {
      amount: amount,
      onPickerChange: handler
    })); //click first disabled

    wrapper.find(_Label.default).at(0).simulate('click');
    expect(handler).not.toHaveBeenCalled(); //click controlDown disabled

    wrapper.find(_Control.default).at(0).simulate('click');
    expect(handler).not.toHaveBeenCalled(); //click 3

    wrapper.find(_Picker.default).at(2).simulate('click');
    expect(handler).toHaveBeenLastCalledWith(3); //click 3

    wrapper.find(_Picker.default).at(0).simulate('click');
    expect(handler).toHaveBeenCalledTimes(1); //click first

    wrapper.find(_Label.default).at(0).simulate('click');
    expect(handler).toHaveBeenLastCalledWith(1); //click last

    wrapper.find(_Label.default).at(1).simulate('click');
    expect(handler).toHaveBeenLastCalledWith(12); //click last disabled

    wrapper.find(_Label.default).at(1).simulate('click');
    expect(handler).toHaveBeenCalledTimes(3); //click controlUp disabled

    wrapper.find(_Control.default).at(1).simulate('click');
    expect(handler).toHaveBeenCalledTimes(3); //change input to 5

    wrapper.find(_Input.default).simulate('change', {
      target: {
        value: 5
      }
    });
    expect(handler).toHaveBeenLastCalledWith(5); //change input to 5 (not invoke handler)

    wrapper.find(_Input.default).simulate('change', {
      target: {
        value: 5
      }
    });
    expect(handler).toHaveBeenCalledTimes(4); //click controlDown (-1)

    wrapper.find(_Control.default).at(0).simulate('click');
    expect(handler).toHaveBeenLastCalledWith(4); //click controlUp (+1)

    wrapper.find(_Control.default).at(1).simulate('click');
    expect(handler).toHaveBeenLastCalledWith(5);
  });
  it('should toggle direction strategy, when same picker clicked', function () {
    var amount = 120;
    var wrapper = (0, _enzyme.mount)(_react.default.createElement(_Paginator.default, {
      amount: amount
    })); //1,2,3...12 |-> click 3 |-> 3,4,5...12

    wrapper.find(_Picker.default).at(2).simulate('click');
    expect(wrapper.find(_Picker.default).at(0).contains(3)).toBeTruthy(); //3,4,5...12 |-> click 3 |-> 1,2,3...12

    wrapper.find(_Picker.default).at(0).simulate('click');
    expect(wrapper.find(_Picker.default).at(2).contains(3)).toBeTruthy();
  });
});