"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.producePickerMap = exports.addValues = exports.addValue = exports.calculateIndexes = exports.calculateIndexesDown = exports.calculateIndexesUp = exports.createSequence = exports.calculateVisiblePickers = exports.calculateAllAvailablePickers = exports.isValueInvalid = void 0;

var _functional = require("./functional");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var isValueInvalid = function isValueInvalid(value) {
  return !value || value < 1;
};

exports.isValueInvalid = isValueInvalid;

var calculateAllAvailablePickers = function calculateAllAvailablePickers(all, perPage) {
  return isValueInvalid(all) || isValueInvalid(perPage) ? 0 : Math.ceil(all / perPage);
};

exports.calculateAllAvailablePickers = calculateAllAvailablePickers;

var calculateVisiblePickers = function calculateVisiblePickers(all, toShow) {
  if (isValueInvalid(all) || isValueInvalid(toShow)) {
    return 0;
  }

  if (all >= toShow) {
    return toShow;
  }

  return all;
};

exports.calculateVisiblePickers = calculateVisiblePickers;

var createSequence = function createSequence(from, to) {
  var step = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;

  if (isNaN(parseInt(from)) || isNaN(parseInt(to))) {
    return [];
  }

  var max = Math.max(from, to);
  var min = Math.min(from, to);

  var getBreakCase = function getBreakCase(from) {
    return step > 0 && from > max || step < 0 && from < min;
  };

  var makeStep = function makeStep() {
    return from = from + step;
  };

  var result = [from];

  while (!getBreakCase(makeStep())) {
    result.push(from);
  }

  return result;
};

exports.createSequence = createSequence;

var calculateIndexesUp = function calculateIndexesUp(currentIndex, indexCount, lastIndex) {
  if (currentIndex > lastIndex) {
    throw new Error('Current index can`t be more then last index');
  }

  var withoutLast = currentIndex + indexCount < lastIndex;

  if (!withoutLast) {
    return createSequence(lastIndex - indexCount, lastIndex);
  }

  return createSequence(currentIndex, indexCount + (currentIndex - 1));
};

exports.calculateIndexesUp = calculateIndexesUp;

var calculateIndexesDown = function calculateIndexesDown(currentIndex, indexCount, lastIndex) {
  if (currentIndex - indexCount <= 0) {
    return createSequence(1, indexCount);
  }

  var from = currentIndex - (indexCount - 1);
  return calculateIndexesUp(from, indexCount, lastIndex);
};

exports.calculateIndexesDown = calculateIndexesDown;

var calculateIndexes = function calculateIndexes(_ref) {
  var currentIndex = _ref.currentIndex,
      up = _ref.up,
      down = _ref.down,
      all = _ref.all,
      visibleAmount = _ref.visibleAmount;
  var calcIndexUp = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : calculateIndexesUp;
  var calIndexDown = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : calculateIndexesDown;

  if (up && down) {
    throw new Error("only one direction should be provided");
  }

  if (isValueInvalid(currentIndex) || isValueInvalid(all) || isValueInvalid(visibleAmount)) {
    return [];
  }

  if (visibleAmount <= 2) {
    return createSequence(1, visibleAmount);
  }

  var args = [currentIndex, visibleAmount - 1, all];

  if (!up && !down) {
    up = true;
  }

  return up ? calcIndexUp.apply(void 0, args) : calIndexDown.apply(void 0, args);
};

exports.calculateIndexes = calculateIndexes;

var addValue = function addValue(value) {
  return function () {
    var arr = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    return value ? _toConsumableArray(arr).concat([value]) : arr;
  };
};

exports.addValue = addValue;

var addValues = function addValues() {
  var values = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  return function () {
    var arr = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    return values.length ? arr.concat(values) : arr;
  };
};

exports.addValues = addValues;

var producePickerMap = function producePickerMap() {
  var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      withLast = _ref2.withLast,
      currentIndex = _ref2.currentIndex,
      lastIndex = _ref2.lastIndex,
      _ref2$indexes = _ref2.indexes,
      indexes = _ref2$indexes === void 0 ? [] : _ref2$indexes,
      _ref2$labels = _ref2.labels,
      labels = _ref2$labels === void 0 ? {} : _ref2$labels,
      _ref2$controls = _ref2.controls,
      controls = _ref2$controls === void 0 ? {} : _ref2$controls,
      delimeter = _ref2.delimeter;

  (0, _functional.assert)(lastIndex, 'last index should be provided');
  return (0, _functional.pipe)(addValue(labels.firstLabel), addValue(controls.controlDown), addValues(indexes), addValue(withLast && delimeter), addValue(withLast && lastIndex), addValue(controls.controlUp), addValue(labels.lastLabel))([]).map(function (value) {
    var isControlUp = value === controls.controlUp;
    var isControlDown = value === controls.controlDown;
    var isFirsLabel = value === labels.firstLabel;
    var isLastLabel = value === labels.lastLabel;
    var isFirst = currentIndex === 1;
    var isLast = currentIndex === lastIndex;
    var disableFirst = isControlDown || isFirsLabel;
    var disableLast = isControlUp || isLastLabel;
    var disabled = disableFirst && isFirst || disableLast && isLast;
    var picked = value === currentIndex;
    return {
      value: value,
      disabled: disabled,
      picked: picked
    };
  });
};

exports.producePickerMap = producePickerMap;